import subprocess

REQUIRED_COMMAND = [
    'hugin_executor',
    'pto_gen',
    'cpfind',
    'cpclean',
    'linefind',
    'autooptimiser',
    'celeste_standalone',
    'pano_modify',
    'convert',
]


def argsToString(args):
    to_string = ''
    for each in args:
        to_string = to_string + ' ' + each
    to_string = to_string.strip()
    return to_string


class Command(object):
    def __init__(self, name, args):
        if name not in REQUIRED_COMMAND:
            raise CommandError(
                '400', 'Command [' + name + '] cannot be recognized!')
        self.name = name
        self.args = args

    def run(self):
        self.args.insert(0, self.name)
        to_be_executed = argsToString(self.args)
        try:
            outputs = subprocess.check_output(
                to_be_executed, stderr=subprocess.STDOUT, stdin=subprocess.PIPE, shell=True)
        except subprocess.CalledProcessError as e:
            out_bytes = e.output
            out_code = str(e.returncode)
            raise CommandError(out_code, out_bytes)
        finally:
            print('[pystitch]: ' + to_be_executed + ' executed!')


class CommandError(RuntimeError):
    def __init__(self, status, message):
        self.status = status
        self.message = message
        self.args = (status, message)


def warmUp():
    for each in REQUIRED_COMMAND:
        try:
            outputs = subprocess.check_output([each], stderr=subprocess.STDOUT)
        except FileNotFoundError as e:
            out_bytes = 'Command ' + each + 'is required but not found in your $PATH'
            out_code = '404'
            raise CommandError(out_code, out_bytes)
        except subprocess.CalledProcessError as e:
            pass
    print('[pystitch]: Test Finished!')

def cleanIntermediateFile():
    print('[pystitch]: Intermediate Files Deleted')


from pystitch.process import automatic
from pystitch.pystitch import warmUp

import os
import time

cwd = os.getcwd()

if __name__=='__main__':
    warmUp()
    start = time.time()
    automatic(os.path.join(cwd, 'data2'),'jpg','project.pto')
    end = time.time()
    print('[pystitch]: Finished in :' + str(end-start) + 's')


import setuptools

setuptools.setup(
    name="stitchpy",
    version="0.0.4",
    author="Xiaozhe Yao",
    author_email="xiaozhe.yaoi@gmail.com",
    description="Python Binding for Pano Tools",
    url="https://autoai.org",
    packages=setuptools.find_packages(),
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development",
    ],
    project_urls={
        "Bug Tracker": "https://github.com/xzyaoi/pystitch/issues",
        "Documentation": "https://cvpm.autoai.org",
        "Source Code": "https://github.com/xzyaoi/pystitch",
    },
)


from pystitch.pystitch import Command, CommandError
from os import path


def automatic(directory='.', image_suffix='.jpg', project_suffix='pystitch',project_name='project.pto'):
    # Generate Project
    commands = []
    commands.append(
        Command('pto_gen', ['-o', project_name, directory+'/*.' + image_suffix]))
    # Generate Control Points
    commands.append(
        Command('cpfind', ['--multirow', '-o', project_name, project_name]))
    # Pruning Control Points
    commands.append(Command('celeste_standalone', [
                    '-i', project_name, '-o', project_name]))
    # Clean
    commands.append(Command('cpclean', ['-o', project_name, project_name]))
    # Find Vertical Lines
    commands.append(Command('linefind', ['-o', project_name, project_name]))
    # Optimising Positions and Geometry
    commands.append(
        Command('autooptimiser', ['-a', '-l', '-s', '-m', '-o', project_name, project_name]))
    # Optimal Crop and Optimal Size
    commands.append(Command('pano_modify', [
                    '-o', project_name, '--center','--fov=AUTO' ,'--canvas=AUTO', '--ldr-file=JPG', project_name]))
    commands.append(Command('hugin_executor', [
                    '--stitching', '--prefix='+ project_suffix, project_name]))
    for each in commands:
        try:
            each.run()
        except CommandError as e:
            print('[pystitch][' + str(e.status) + ']: ' + str(e.message, 'utf-8'))
            break
        
